@@ -52,6 +52,8 @@
 #include <osmocom/vty/command.h>
 #include <osmocom/vty/ports.h>
 
+#include <osmocom/mgcp_client/mgcp_client.h>
+
 #include <osmocom/netif/stream.h>
 
 #include <osmocom/ranap/ranap_common.h>
@@ -86,12 +88,19 @@ static struct hnb_gw *hnb_gw_create(void *ctx)
 	gw->config.iuh_local_port = IUH_DEFAULT_SCTP_PORT;
 	gw->config.log_prefix_hnb_id = true;
 
+	/* No limit by default, always include the initial RANAP message in the SCCP CR towards the CN.
+	 * 999999 is the maximum value in hnbgw_vty.c */
+	gw->config.max_sccp_cr_payload_len = 999999;
+
 	gw->next_ue_ctx_id = 23;
 	INIT_LLIST_HEAD(&gw->hnb_list);
 	INIT_LLIST_HEAD(&gw->ue_list);
 
 	context_map_init(gw);
 
+	gw->config.mgcp_client = talloc_zero(tall_hnb_ctx, struct mgcp_client_conf);
+	mgcp_client_conf_init(gw->config.mgcp_client);
+
 	return gw;
 }
 
@@ -338,6 +347,11 @@ void hnb_context_release(struct hnb_context *ctx)
 	talloc_free(ctx);
 }
 
+bool hnbgw_requires_empty_sccp_cr(struct hnb_gw *gw, unsigned int ranap_msg_len)
+{
+	return ranap_msg_len > gw->config.max_sccp_cr_payload_len;
+}
+
 /*! call-back when the listen FD has something to read */
 static int accept_cb(struct osmo_stream_srv_link *srv, int fd)
 {
@@ -372,6 +386,11 @@ static const struct log_info_cat log_cat[] = {
 		.color = "",
 		.description = "RAN Application Part",
 	},
+	[DMGW] = {
+		.name = "DMGW", .loglevel = LOGL_NOTICE, .enabled = 1,
+		.color = "\033[1;33m",
+		.description = "Media Gateway",
+	},
 };
 
 static const struct log_info hnbgw_log_info = {
@@ -681,6 +700,19 @@ int main(int argc, char **argv)
 	}
 	g_hnb_gw->iuh = srv;
 
+	/* Initialize and connect MGCP client. */
+	g_hnb_gw->mgcp_client = mgcp_client_init(tall_hnb_ctx, g_hnb_gw->config.mgcp_client);
+	if (!g_hnb_gw->mgcp_client) {
+		LOGP(DMGW, LOGL_ERROR, "MGW client initalization failed\n");
+		return -EINVAL;
+	}
+	if (mgcp_client_connect(g_hnb_gw->mgcp_client)) {
+		LOGP(DMGW, LOGL_ERROR, "MGW connect failed at (%s:%u)\n",
+		     g_hnb_gw->config.mgcp_client->remote_addr,
+		     g_hnb_gw->config.mgcp_client->remote_port);
+		return -EINVAL;
+	}
+
 	if (hnbgw_cmdline_config.daemonize) {
 		rc = osmo_daemonize();
 		if (rc < 0) {
@@ -690,7 +722,7 @@ int main(int argc, char **argv)
 	}
 
 	while (1) {
-		rc = osmo_select_main(0);
+		rc = osmo_select_main_ctx(0);
 		if (rc < 0)
 			exit(3);
 	}
