@@ -34,6 +34,11 @@
 #include <osmocom/ranap/ranap_ies_defs.h>
 #include <osmocom/ranap/ranap_msg_factory.h>
 #include <osmocom/hnbgw/context_map.h>
+#include <osmocom/hnbgw/mgw_fsm.h>
+#include <osmocom/ranap/RANAP_ProcedureCode.h>
+#include <osmocom/ranap/ranap_common.h>
+#include <osmocom/ranap/ranap_common_ran.h>
+#include <osmocom/ranap/RANAP_RANAP-PDU.h>
 
 /***********************************************************************
  * Outbound RANAP RESET to CN
@@ -266,7 +271,7 @@ static int handle_cn_ranap(struct hnbgw_cnlink *cnlink, const struct osmo_scu_un
 	int rc;
 
 	memset(pdu, 0, sizeof(*pdu));
-	dec_ret = aper_decode(NULL,&asn_DEF_RANAP_RANAP_PDU, (void **) &pdu,
+	dec_ret = aper_decode(NULL, &asn_DEF_RANAP_RANAP_PDU, (void **) &pdu,
 			      data, len, 0, 0);
 	if (dec_ret.code != RC_OK) {
 		LOGP(DRANAP, LOGL_ERROR, "Error in RANAP ASN.1 decode\n");
@@ -324,18 +329,19 @@ static int handle_cn_conn_conf(struct hnbgw_cnlink *cnlink,
 			       const struct osmo_scu_connect_param *param,
 			       struct osmo_prim_hdr *oph)
 {
-	/* we don't actually need to do anything, as RUA towards the HNB
-	 * doesn't seem to know any confirmations to its CONNECT
-	 * operation */
+	struct osmo_ss7_instance *ss7 = osmo_sccp_get_ss7(cnlink->gw->sccp.client);
+	LOGP(DMAIN, LOGL_DEBUG, "handle_cn_conn_conf() conn_id=%d, addrs: called=%s calling=%s responding=%s\n",
+	     param->conn_id,
+	     osmo_sccp_addr_to_str_c(OTC_SELECT, ss7, &param->called_addr),
+	     osmo_sccp_addr_to_str_c(OTC_SELECT, ss7, &param->calling_addr),
+	     osmo_sccp_addr_to_str_c(OTC_SELECT, ss7, &param->responding_addr));
 
-	LOGP(DMAIN, LOGL_DEBUG, "handle_cn_conn_conf() conn_id=%d\n",
-	     param->conn_id);
-	LOGP(DMAIN, LOGL_DEBUG, "handle_cn_conn_conf() called_addr=%s\n",
-	     inet_ntoa(param->called_addr.ip.v4));
-	LOGP(DMAIN, LOGL_DEBUG, "handle_cn_conn_conf() calling_addr=%s\n",
-	     inet_ntoa(param->calling_addr.ip.v4));
-	LOGP(DMAIN, LOGL_DEBUG, "handle_cn_conn_conf() responding_addr=%s\n",
-	     inet_ntoa(param->responding_addr.ip.v4));
+	/* Nothing needs to happen for RUA, RUA towards the HNB doesn't seem to know any confirmations to its CONNECT
+	 * operation. */
+
+	/* If our initial SCCP CR was sent without data payload, then the initial RANAP message is cached and waiting to
+	 * be sent as soon as the SCCP connection is confirmed. See if that is the case, send cached data. */
+	context_map_send_cached_msg(context_map_by_cn(cnlink, param->conn_id));
 
 	return 0;
 }
@@ -345,10 +351,13 @@ static int handle_cn_data_ind(struct hnbgw_cnlink *cnlink,
 			      struct osmo_prim_hdr *oph)
 {
 	struct hnbgw_context_map *map;
+	ranap_message *message;
+	int rc;
 
-	/* connection-oriented data is always passed transparently
-	 * towards the specific HNB, via a RUA connection identified by
-	 * conn_id */
+	/* Usually connection-oriented data is always passed transparently towards the specific HNB, via a RUA
+	 * connection identified by conn_id. An exception is made for RANAP RAB AssignmentRequest and
+	 * RANAP RAB AssignmentResponse, since those messages contain transport layer information (RTP stream IP/Port),
+	 * which is rewritten by the FSM that controls the co-located media gateway. */
 
 	map = context_map_by_cn(cnlink, param->conn_id);
 	if (!map) {
@@ -356,6 +365,28 @@ static int handle_cn_data_ind(struct hnbgw_cnlink *cnlink,
 		return 0;
 	}
 
+	/* Intercept RAB Assignment Request, Setup MGW FSM */
+	if (!map->is_ps) {
+		message = talloc_zero(map, ranap_message);
+		rc = ranap_ran_rx_co_decode(map, message, msgb_l2(oph->msg), msgb_l2len(oph->msg));
+
+		if (rc == 0) {
+			switch (message->procedureCode) {
+			case RANAP_ProcedureCode_id_RAB_Assignment:
+				/* mgw_fsm_alloc_and_handle_rab_ass_req() takes ownership of (ranap) message */
+				return handle_rab_ass_req(map, oph, message);
+			case RANAP_ProcedureCode_id_Iu_Release:
+				/* Any IU Release will terminate the MGW FSM, the message itsself is not passed to the
+				 * FSM code. It is just forwarded normally by the rua_tx_dt() call below. */
+				mgw_fsm_release(map);
+				break;
+			}
+			ranap_ran_rx_co_free(message);
+		}
+
+		talloc_free(message);
+	}
+
 	return rua_tx_dt(map->hnb_ctx, map->is_ps, map->rua_ctx_id,
 			 msgb_l2(oph->msg), msgb_l2len(oph->msg));
 }
